@@ -149,7 +149,7 @@ get_fid_event_fd (const struct fanotify_event_metadata *data)
 
     /* get affected file fd from fanotify_event_info_fid */
     fd = open_by_handle_at (get_mount_id ((const fsid_t *) &fid->fsid),
-                            (struct file_handle *) fid->handle, O_RDONLY);
+                            (struct file_handle *) fid->handle, O_RDONLY|O_NONBLOCK|O_LARGEFILE|O_PATH);
     /* ignore ESTALE for deleted fds between the notification and handling it */
     if (fd < 0 && errno != ESTALE)
         warn ("open_by_handle_at");
@@ -308,7 +308,7 @@ print_event (const struct fanotify_event_metadata *data,
     } else if (option_timestamp == 2) {
         printf ("%li.%06li ", event_time->tv_sec, event_time->tv_usec);
     }
-    printf ("%s(%i): %s %s\n", procname[0] == '\0' ? "unknown" : procname, data->pid, mask2str (data->mask), pathname);
+    printf ("%s(%i): %-3s %s\n", procname[0] == '\0' ? "unknown" : procname, data->pid, mask2str (data->mask), pathname);
 }
 
 static void
@@ -378,8 +378,11 @@ setup_fanotify (int fan_fd)
          * are virtual and do not actually cause disk access. */
         if (mount->mnt_fsname == NULL || access (mount->mnt_fsname, F_OK) != 0 ||
             mount->mnt_fsname[0] != '/') {
-            debug ("ignore: fsname: %s dir: %s type: %s", mount->mnt_fsname, mount->mnt_dir, mount->mnt_type);
-            continue;
+            /* zfs mount point don't start with a "/" so allow them anyway */
+            if (strcmp(mount->mnt_type, "zfs") != 0) {
+                debug ("ignore: fsname: %s dir: %s type: %s", mount->mnt_fsname, mount->mnt_dir, mount->mnt_type);
+                continue;
+            }
         }
 
         /* root dir already added above */
@@ -653,4 +656,3 @@ main (int argc, char** argv)
 
     return 0;
 }
-
